@@ -1,7 +1,7 @@
 /************************************************************************
 * db.c
 * nodau console note taker
-* Copyright (C) Lisa Milne 2010-2012 <lisa@ltmnet.com>
+* Copyright (C) Lisa Milne 2010-2013 <lisa@ltmnet.com>
 *
 * db.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
@@ -252,22 +252,29 @@ int db_result_free(sql_result *result)
 /* update an existing note */
 int db_update(char* name, char* value)
 {
-	char sql[512];
+	char* sql;
+	int r;
 	/* create the sql statement using the name/text for this note
 	 * if it's meant to be encrypted, then crypt_key will be set */
 	if (crypt_key) {
 		value = note_encrypt(value,crypt_key);
-		sprintf(sql, "UPDATE nodau set text='%s' , encrypted='true' WHERE name='%s'", value, name);
+		r = asprintf(&sql, "UPDATE nodau set text='%s' , encrypted='true' WHERE name='%s'", value, name);
+		free(value);
+		if (r < 0)
+			return 1;
 	}else{
-		sprintf(sql, "UPDATE nodau set text='%s' , encrypted='false' WHERE name='%s'", value, name);
+		if (asprintf(&sql, "UPDATE nodau set text='%s' , encrypted='false' WHERE name='%s'", value, name) < 0)
+			return 1;
 	}
 
 	/* do it */
-	return sqlite3_exec(db, sql, NULL, 0, &error_msg);
+	r = sqlite3_exec(db, sql, NULL, 0, &error_msg);
+	free(sql);
+	return r;
 }
 
 /* list notes according to search criteria */
-void db_list(char* search)
+int db_list(char* search)
 {
 	sql_result *res;
 	int i;
@@ -282,7 +289,7 @@ void db_list(char* search)
 		if (res->num_rows == 0) {
 			printf("No notes to list\n");
 			db_result_free(res);
-			return;
+			return 0;
 		}
 	}else{
 		/* first try a name search */
@@ -309,7 +316,7 @@ void db_list(char* search)
 		/* nothing there */
 		if (!res || !res->num_rows || !res->num_cols) {
 			printf("No notes match '%s'\n",search);
-			return;
+			return 0;
 		}
 	}
 
@@ -320,24 +327,31 @@ void db_list(char* search)
 
 	/* free the result */
 	db_result_free(res);
+
+	return 0;
 }
 
 /* open an existing note */
-void db_edit(char* search)
+int db_edit(char* search)
 {
 	char* date;
 	char* name;
 	char* text;
 	char* crypt;
+	int r;
 	/* get the note by name */
 	sql_result *result;
 	result = db_get("SELECT * FROM nodau WHERE name = '%s'",search);
 
 	/* nothing there */
 	if (result->num_rows == 0) {
-		printf("No notes match '%s'\n",search);
 		db_result_free(result);
-		return;
+		if (config_read("edit_autocreate","false")) {
+			printf("No notes match '%s'\n",search);
+		}else{
+			 return db_new(search);
+		}
+		return 0;
 	}
 
 	/* get the data */
@@ -351,32 +365,34 @@ void db_edit(char* search)
 		crypt = crypt_get_key();
 		text = note_decrypt(text,crypt);
 		if (!text)
-			return;
+			return 1;
 	}
 
 	/* edit the note */
-	edit(name, date, text);
+	r = edit(name, date, text);
 
 	/* free the result */
 	db_result_free(result);
+
+	return r;
 }
 
-/* show an existing note */
-void db_show(char* search)
+/* append data from stdin to a note */
+int db_append(char* search)
 {
 	char* date;
 	char* name;
 	char* text;
 	char* crypt;
+	int r;
 	/* get the note by name */
 	sql_result *result;
 	result = db_get("SELECT * FROM nodau WHERE name = '%s'",search);
 
 	/* nothing there */
 	if (result->num_rows == 0) {
-		printf("No notes match '%s'\n",search);
 		db_result_free(result);
-		return;
+		return db_new(search);
 	}
 
 	/* get the data */
@@ -390,7 +406,48 @@ void db_show(char* search)
 		crypt = crypt_get_key();
 		text = note_decrypt(text,crypt);
 		if (!text)
-			return;
+			return 1;
+	}
+
+	/* edit the note */
+	r = edit_stdin(name, date, text,1);
+
+	/* free the result */
+	db_result_free(result);
+
+	return r;
+}
+
+/* show an existing note */
+int db_show(char* search)
+{
+	char* date;
+	char* name;
+	char* text;
+	char* crypt;
+	/* get the note by name */
+	sql_result *result;
+	result = db_get("SELECT * FROM nodau WHERE name = '%s'",search);
+
+	/* nothing there */
+	if (result->num_rows == 0) {
+		printf("No notes match '%s'\n",search);
+		db_result_free(result);
+		return 0;
+	}
+
+	/* get the data */
+	date = db_gettime(result->data[COLUMN(0,COL_DATE)]);
+	name = result->data[COLUMN(0,COL_NAME)];
+	text = result->data[COLUMN(0,COL_TEXT)];
+	crypt = result->data[COLUMN(0,COL_CRYPT)];
+
+	/* get the passphrase if it's encrypted */
+	if (!strcmp(crypt,"true")) {
+		crypt = crypt_get_key();
+		text = note_decrypt(text,crypt);
+		if (!text)
+			return 1;
 	}
 
 	/* display the note */
@@ -398,10 +455,12 @@ void db_show(char* search)
 
 	/* free the result */
 	db_result_free(result);
+
+	return 0;
 }
 
 /* delete notes */
-void db_del(char* search)
+int db_del(char* search)
 {
 	char sql[512];
 	unsigned int date = 0;
@@ -432,7 +491,7 @@ void db_del(char* search)
 	/* or print an error */
 	}else{
 		printf("No notes matches '%s'\n",search);
-		return;
+		return 0;
 	}
 
 	/* run the statement */
@@ -440,10 +499,12 @@ void db_del(char* search)
 
 	/* free the earlier result */
 	db_result_free(result);
+
+	return 0;
 }
 
 /* create a new note */
-void db_new(char* search)
+int db_new(char* search)
 {
 	/* search by name */
 	sql_result *result;
@@ -454,7 +515,7 @@ void db_new(char* search)
 		if (result->num_rows) {
 			printf("There is already a note called '%s'\n",search);
 			db_result_free(result);
-			return;
+			return 1;
 		}
 
 		/* free the search result */
@@ -468,15 +529,16 @@ void db_new(char* search)
 		printf("%s\n",error_msg);
 
 	/* open for editing */
-	db_edit(search);
+	return db_edit(search);
 }
 
 /* encrypt an existing note, or create a new encrypted note */
-void db_encrypt(char* search)
+int db_encrypt(char* search)
 {
 	/* search by name */
 	sql_result *result;
 	char* crypt;
+	int r;
 	result = db_get("SELECT * FROM nodau WHERE name = '%s'",search);
 
 	/* there's already a note with that name */
@@ -492,12 +554,12 @@ void db_encrypt(char* search)
 		/* encrypt it if it's not already */
 		if (!strcmp(crypt,"false")) {
 			crypt = crypt_get_key();
-			db_update(name,text);
+			r = db_update(name,text);
 		}else{
 			printf("Note '%s' is already encrypted\n",search);
 		}
 		db_result_free(result);
-		return;
+		return r;
 	}
 
 	/* free the search result */
@@ -507,19 +569,20 @@ void db_encrypt(char* search)
 	db_insert(search,"new entry");
 
 	if (error_msg)
-		printf("%s\n",error_msg);
+		fprintf(stderr,"%s\n",error_msg);
 
 	crypt = crypt_get_key();
 	/* open for editing */
-	db_edit(search);
+	return db_edit(search);
 }
 
 
 /* decrypt an existing note or create a new encrypted note */
-void db_decrypt(char* search)
+int db_decrypt(char* search)
 {
 	/* search by name */
 	sql_result *result;
+	int r;
 	result = db_get("SELECT * FROM nodau WHERE name = '%s'",search);
 
 	/* found the note */
@@ -537,18 +600,21 @@ void db_decrypt(char* search)
 			crypt = crypt_get_key();
 			t = note_decrypt(text,crypt);
 			if (!t)
-				return;
+				return 1;
 			free(crypt_key);
 			crypt_key = NULL;
-			db_update(search,t);
+			r = db_update(search,t);
 			db_result_free(result);
+			return r;
 		}else{
 			printf("Note '%s' is not encrypted\n",search);
 			db_result_free(result);
 		}
-		return;
+		return 0;
 	}
 
 	printf("No notes matches '%s'\n",search);
 	db_result_free(result);
+
+	return 0;
 }
