@@ -1,7 +1,7 @@
 /************************************************************************
 * edit.c
 * nodau console note taker
-* Copyright (C) Lisa Milne 2010-2012 <lisa@ltmnet.com>
+* Copyright (C) Lisa Milne 2010-2013 <lisa@ltmnet.com>
 *
 * edit.c is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
@@ -59,7 +59,7 @@ static void draw(char* data)
 }
 
 /* edit a note with the builtin editor */
-static void edit_builtin(char* name, char* date, char* data)
+static int edit_builtin(char* name, char* date, char* data)
 {
 	char buffer[256];
 	int bl;
@@ -140,42 +140,51 @@ static void edit_builtin(char* name, char* date, char* data)
 	endwin();
 
 	/* save the note */
-	db_update(name,buffer);
+	if (!db_update(name,buffer))
+		return 1;
 
 	/* let the user know */
 	printf("%s saved\n",name);
+
+	return 0;
 }
 
 /* edit with an external editor */
-int edit_ext(char* editor, char* name, char* date, char* data)
+static int edit_ext(char* editor, char* name, char* date, char* data)
 {
-	FILE *f;
+	int fd;
 	int st;
 	int sz;
 	char* b;
 	char* l;
 	char buff[512];
 	pid_t pid;
-	sprintf(buff,"/tmp/nodau.%d",(int)time(NULL));
+
+	strcpy(buff,"/tmp/nodau.XXXXXX");
+	fd = mkstemp(buff);
+
+	if (fd < 0)
+		return 1;
 
 	pid = fork();
 
 	if (pid < 0) {
 		return 1;
 	}else if (pid) {
+		close(fd);
 		waitpid(pid,&st,0);
 		if (!st) {
-			if ((f = fopen(buff,"r")) == NULL)
+			if ((fd = open(buff,O_RDONLY)) < 0)
 				return 1;
 			/* find the file length */
-			fseek(f,0,SEEK_END);
-			sz = ftell(f);
-			fseek(f,0,SEEK_SET);
+			sz = lseek(fd,0,SEEK_END);
+			lseek(fd,0,SEEK_SET);
 			if (sz) {
 				/* load the note into memory */
 				b = alloca(sz+1);
-				fread(b,1,sz,f);
-				fclose(f);
+				if (sz != read(fd,b,sz))
+					return 1;
+				close(fd);
 				/* delete the file */
 				remove(buff);
 				b[sz] = 0;
@@ -184,7 +193,8 @@ int edit_ext(char* editor, char* name, char* date, char* data)
 				if (l) {
 					/* save the note */
 					l += 6;
-					db_update(name,l);
+					if (db_update(name,l))
+						return 1;
 
 					/* let the user know */
 					printf("%s saved\n",name);
@@ -194,19 +204,20 @@ int edit_ext(char* editor, char* name, char* date, char* data)
 		return st;
 	}
 
-	if ((f = fopen(buff,"w+")) == NULL)
-		exit(1);
+	sz = strlen(name)+strlen(date)+strlen(data)+50;
+	b = alloca(sz);
 
 	/* insert data into file */
-	fprintf(
-		f,
+	sz = sprintf(
+		b,
 		"%s (%s)\nText above this line is ignored\n-----\n%s",
 		name,
 		date,
 		data
 	);
-	fflush(f);
-	fclose(f);
+	write(fd,b,sz);
+	fsync(fd);
+	close(fd);
 
 	st = execl(editor,editor,buff,(char*)NULL);
 
@@ -217,24 +228,85 @@ int edit_ext(char* editor, char* name, char* date, char* data)
 	return 1;
 }
 
+/* edit a note using data piped from stdin */
+int edit_stdin(char* name, char* date, char* data, int append)
+{
+	char buff[1024];
+	int l;
+	int s;
+	int r;
+	char* d;
+	char* b;
+
+	/* get the initial buffer size */
+	l = strlen(data);
+	if (l < 512) {
+		s = 512;
+	}else{
+		s = l*2;
+	}
+
+	d = malloc(s);
+	if (!d)
+		return 1;
+
+	/* for append mode copy the old data to the start of the buffer */
+	if (append && strcmp(data,"new entry")) {
+		strcpy(d,data);
+	}else{
+		l = 0;
+	}
+
+	/* read it in */
+	while ((r = read(STDIN_FILENO,buff,1024)) > 0) {
+		/* extend the buffer as necessary */
+		if (l+r+1 > s) {
+			s = l+r+512;
+			b = realloc(d,s);
+			if (!b)
+				return 1;
+			d = b;
+		}
+		memcpy(d+l,buff,r);
+		l += r;
+	}
+
+	/* make sure there's room for the nul byte */
+	if (l+1 > s) {
+		s = l+1;
+		b = realloc(d,s);
+		if (!b)
+			return 1;
+		d = b;
+	}
+
+	d[l] = 0;
+
+	/* done */
+	return db_update(name,d);
+}
+
 /* edit a note */
-void edit(char* name, char* date, char* data)
+int edit(char* name, char* date, char* data)
 {
 	char* ed;
-	char* pt = getenv("PATH");
+	char* pt;
 	char* editor;
 	char* p = NULL;
 	struct stat st;
 
+	if (!isatty(STDIN_FILENO))
+		return edit_stdin(name,date,data,0);
+
+	pt = getenv("PATH");
+
 	ed = config_read("external_editor",NULL);
 	if (!ed)
 		ed = getenv("EDITOR");
 
 	/* no editor or no path, use builtin */
-	if (config_read("force_builtin_editor","true") || !ed || (ed[0] != '/' && !pt)) {
-		edit_builtin(name,date,data);
-		return;
-	}
+	if (config_read("force_builtin_editor","true") || !ed || (ed[0] != '/' && !pt))
+		return edit_builtin(name,date,data);
 
 	/* find the executable */
 	if (ed[0] == '/') {
@@ -262,8 +334,8 @@ void edit(char* name, char* date, char* data)
 	}
 
 	/* no executable, or fails to run, use builtin */
-	if (!p || edit_ext(editor,name,date,data)) {
-		edit_builtin(name,date,data);
-		return;
-	}
+	if (!p || edit_ext(editor,name,date,data))
+		return edit_builtin(name,date,data);
+
+	return 0;
 }
